package com.jprofiler.api.agent.probe;

import java.util.ArrayList;
import java.util.List;

/**
 * A stack that saves objects in thread local data structures.
 * This utility class is important for calculating payload timing where you need to save payload infos
 * in the {@link InterceptorProbe#interceptionEnter(InterceptorContext, Object, Class, String, String, String, Object[])}
 * method and retrieve them in the {@link InterceptorProbe#interceptionExit(InterceptorContext, Class, String, String, String, Object)} method
 * of an interceptor.
 * <p>
 *     Usually, you can use the {@link InterceptorContext#push(PayloadInfo)} and {@link InterceptorContext#pop()} methods
 *     which use such a thread local stack. Use this class only if you have to maintain more that ony such stack in a single probe.
 * </p>
 */
public class ThreadLocalStack<T> {
    private ThreadLocal<ArrayList<T>> stack = new ThreadLocal<ArrayList<T>>() {
        @Override
        protected ArrayList<T> initialValue() {
            return new ArrayList<T>();
        }
    };

    /**
     * Pushes an object onto the thread-local stack.
     * @param object the object
     */
    public void push(T object) {
        stack.get().add(object);
    }

    /**
     * Pops an object from the thread-local stack.
     * @return the object.
     */
    public T pop() {
        final List<T> list = stack.get();
        return list.remove(list.size() - 1);
    }

    /**
     * Retrieves the object on top of the thread-local stack.
     * @return the object.
     */
    public T peek() {
        final List<T> list = stack.get();
        return list.get(list.size() - 1);
    }

    /** Returns the number of objects currently on the thread-local stack.
     *
     * @return the number of objects
     */
    public int size() {
        return stack.get().size();
    }

    /** Returns the stack as a list.
     *
     * @return a list
     */
    public List<T> getList() {
        return stack.get();
    }

    /**
     * Checks if the thread-local stack is empty.
     * @return if the stack is empty or not
     */
    public boolean isEmpty() {
        return size() == 0;
    }

}
